from kivy.utils import platform
from kivy.logger import Logger
import webbrowser
if platform == "android":
    try:
        #from kivy.core.window import Window
        from jnius import autoclass, cast, JavaException,PythonJavaClass,java_method
        from android.runnable import run_on_ui_thread
        from android.permissions import request_permissions, Permission
        from android import activity
        #A
        AndroidActivity = autoclass('android.app.Activity')
        ActivityInfo = autoclass('android.content.pm.ActivityInfo')
        AudioManager = autoclass('android.media.AudioManager')
        ArrayList = autoclass('java.util.ArrayList')
        
        #B
        BatteryManager = autoclass("android.os.BatteryManager")
        BitmapFactory = autoclass('android.graphics.BitmapFactory')
        Build = autoclass("android.os.Build")
        Bundle = autoclass('android.os.Bundle')

        #C

        Camera = autoclass('android.hardware.Camera')
        

        Color = autoclass("android.graphics.Color")
        Context = autoclass('android.content.Context')
        ConnectivityManager = autoclass('android.net.ConnectivityManager')
        Configuration = autoclass("android.content.res.Configuration")
        #D
        DisplayMetrics = autoclass('android.util.DisplayMetrics')
        
        #E
        Environment = autoclass("android.os.Environment")
        #F
        File = autoclass('java.io.File')
        #G
        Gravity=autoclass('android.view.Gravity')
        #I
        Intent = autoclass('android.content.Intent')
        IntentFilter = autoclass("android.content.IntentFilter")
        #J
        #K
        KeyEvent = autoclass('android.view.KeyEvent')

        #L
        Locale = autoclass('java.util.Locale')
        LinearLayout = autoclass('android.widget.LinearLayout')
        
        #M
        MediaController=autoclass('android.widget.MediaController')
        MediaPlayer = autoclass('android.media.MediaPlayer')
        MediaRecorder = autoclass('android.media.MediaRecorder')
        
        #N
        #O
        
        #P
        PythonActivity = autoclass('org.kivy.android.PythonActivity')
        PackageManager = autoclass('android.content.pm.PackageManager')
        Point = autoclass("android.graphics.Point")

        #R
        Rect = autoclass('android.graphics.Rect')()
        RecognizerIntent = autoclass('android.speech.RecognizerIntent')
        RecognitionListener = autoclass('android.speech.RecognitionListener')
        
        Runtime = autoclass('java.lang.Runtime')

        #S
        Settings = autoclass('android.provider.Settings')
        SpeechRecognizer = autoclass('android.speech.SpeechRecognizer')
        String = autoclass("java.lang.String")
        StrictMode = autoclass('android.os.StrictMode')
        StatFs = autoclass("android.os.StatFs")
        SurfaceTexture = autoclass("android.graphics.SurfaceTexture")
        
        #T
        TextToSpeech = autoclass('android.speech.tts.TextToSpeech')
        Toast = autoclass('android.widget.Toast')
        #U
        Uri = autoclass('android.net.Uri')
        URLConnection = autoclass("java.net.URLConnection")
        #V
        
        
        View = autoclass('android.view.View')
        ViewGroup = autoclass('android.view.ViewGroup')
        VideoView=autoclass('android.widget.VideoView')
        #W
        
        WallpaperManager = autoclass('android.app.WallpaperManager')
        WebView = autoclass('android.webkit.WebView')
        WebViewClient = autoclass('android.webkit.WebViewClient')
        Window=autoclass('android.view.Window')
        



        ######Nested classes 
        # N used to indicate nested class
        #A
        ActivityManagerNMemoryInfo = autoclass('android.app.ActivityManager$MemoryInfo')
        
        #B
        BuildNVERSION = autoclass('android.os.Build$VERSION')
        BuildNVERSION_CODES = autoclass("android.os.Build$VERSION_CODES")
        #C
        CameraNCameraInfo=autoclass('android.hardware.Camera$CameraInfo')
        CameraNParameters = autoclass("android.hardware.Camera$Parameters")
        #D
        DownloadNManagerRequest = autoclass("android.app.DownloadManager$Request")
        #E
        #F
        #G
        #H
        #I
        #J
        #K
        #L
        #M
        MediaRecorderNAudioEncoder = autoclass('android.media.MediaRecorder$AudioEncoder')
        MediaRecorderNAudioSource = autoclass('android.media.MediaRecorder$AudioSource')
        MediaRecorderNOutputFormat = autoclass('android.media.MediaRecorder$OutputFormat')
        #N
        #O
        #P
        #Q
        #R
        #S
        SettingsNSystem = autoclass('android.provider.Settings$System')
        #T
        #U
        #V
        ViewGroupNLayoutParams = autoclass('android.view.ViewGroup$LayoutParams')
        
        #W
        WindowManagerNLayoutParams= autoclass('android.view.WindowManager$LayoutParams')
        #X
        #Y
        #Z






        #app's activity
        mActivity = PythonActivity.mActivity
        
        
        
        
        
        
        
        
        
        
       
        
        
        
        


        packages = {
            "whatsapp": "com.whatsapp",
            "facebook": "com.facebook.katana",
            "facebookLite": "com.facebook.lite",
            "oldFacebook": "com.facebook.android",
            "linkedin": "com.linkedin.android",
            "fbMessenger": "com.facebook.orca",
            "fbMessengerLite": "com.facebook.mlite",
            "tiktok": "com.zhiliaoapp.musically",
            "tiktokLite": "com.zhiliaoapp.musically.go",
            "twitter": "com.twitter.android",
            "twitterLite": "com.twitter.android.lite",
            "telegram": "org.telegram.messenger",
            "telegramX": "org.thunderdog.challegram",
            "snapchat": "com.snapchat.android"
        }

    except BaseException:
        Logger.error(
            "Pyandroid: Cannot load classes by Pyjnius. Make sure requirements installed"
        )


    ###########dict of orientation
    orientation_dict={
    'portrait':ActivityInfo.SCREEN_ORIENTATION_PORTRAIT,
    'landscape':ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE,
    'behind':ActivityInfo.SCREEN_ORIENTATION_BEHIND,
    'full_sensor':ActivityInfo.SCREEN_ORIENTATION_FULL_SENSOR,
    'full_user':ActivityInfo.SCREEN_ORIENTATION_FULL_USER,
    'locked':ActivityInfo.SCREEN_ORIENTATION_LOCKED,
    'no_sensor':ActivityInfo.SCREEN_ORIENTATION_NOSENSOR,
    'user': ActivityInfo.SCREEN_ORIENTATION_USER,
    'user_portrait': ActivityInfo.SCREEN_ORIENTATION_USER_PORTRAIT,
    'user_landscape': ActivityInfo.SCREEN_ORIENTATION_USER_LANDSCAPE,
    'unspecified': ActivityInfo.SCREEN_ORIENTATION_UNSPECIFIED,
    'sensor_portrait':ActivityInfo.SCREEN_ORIENTATION_SENSOR_PORTRAIT,
    'sensor_landscape':ActivityInfo.SCREEN_ORIENTATION_SENSOR_LANDSCAPE,
    'sensor':ActivityInfo.SCREEN_ORIENTATION_SENSOR,
    'reverse_portrait':ActivityInfo.SCREEN_ORIENTATION_REVERSE_PORTRAIT,
    'reverse_landscape':ActivityInfo.SCREEN_ORIENTATION_REVERSE_LANDSCAPE,
    }

    def text_to_speech(text, lang=Locale.US):
            tts = TextToSpeech(mActivity, None)
            tts.setLanguage(lang)
            tts.speak(str(text), TextToSpeech.QUEUE_FLUSH, None)

    def launch_app(app_package, app_activity):
        intent = Intent()
        intent.setAction(Intent.ACTION_VIEW)
        intent.setClassName(app_package, app_activity)
        return mActivity.startActivity(intent)

    def toast(message):
        return PythonActivity.toastError(str(message))


    def download_manager(title, description, url, folder, file_name):
        uri = Uri.parse(str(url))
        dm = cast("android.app.DownloadManager", mActivity.getSystemService(Context.DOWNLOAD_SERVICE))
        request = DownloadManagerNRequest(uri)
        request.setTitle(str(title))
        request.setDescription(str(description))
        ########notification bar
        request.setNotificationVisibility(DownloadManagerNRequest.VISIBILITY_VISIBLE)
        request.setDestinationInExternalPublicDir(folder, str(file_name))
        dm.enqueue(request)

    
    @run_on_ui_thread
    def write_settings(sdk_int,version_code):
        if sdk_int>=version_code:

            if SettingsNSystem.canWrite(mActivity.getApplicationContext()):
                print("can write")
            else:
                intent=Intent(Settings.ACTION_MANAGE_WRITE_SETTINGS)
                intent.setData(Uri.parse("package:" + mActivity.getPackageName()))
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                mActivity.startActivity(intent)





    def share_file(path, title='Share', chooser=True, app_package=None, call_playstore=True, error_msg=""):
        path = str(path)
        StrictMode.disableDeathOnFileUriExposure()
        shareIntent = Intent(Intent.ACTION_SEND)
        shareIntent.setType("*/*")
        imageFile = File(path)
        uri = Uri.fromFile(imageFile)
        parcelable = cast('android.os.Parcelable', uri)
        shareIntent.putExtra(Intent.EXTRA_STREAM, parcelable)

        if app_package:
            app_package = packages[app_package] if app_package in packages else None
            try:
                shareIntent.setPackage(String(app_package))
            except JavaException:
                if call_playstore:
                    webbrowser.open(f"http://play.google.com/store/apps/details?id={app_package}")
                toast(error_msg) if error_msg else Logger.error("Pyandroid: Specified Application is unavailable")
                return

        if chooser:
            chooser = Intent.createChooser(shareIntent, String(title))
            mActivity.startActivity(chooser)
        else:
            mActivity.startActivity(shareIntent)



 



from kivy.core.window import Window
from kivy.clock import Clock
from pykivdroid import Intent
from pykivdroid import String as JString
from pykivdroid import activity,mActivity
#Intent = autoclass('android.content.Intent')
#JString = autoclass('java.lang.String')

# Source https://github.com/RobertFlatt/Android-for-Python/storage
# Requires
# READ_EXTERNAL_STORAGE



class AndroidPicker():
    def __init__(self, callback = None, **kwargs):
        self.REQUEST_CODE = 42 
        self.callback=callback
        activity.bind(on_activity_result=self.intent_callback)


    def __del__(self):
        activity.unbind(on_activity_result=self.intent_callback)

    def pick_file(self, MIME_type = '*/*'):
        try:
            self.msg = "Choose a file"
            self.intent = Intent(Intent.ACTION_GET_CONTENT)
            self.intent.setType(MIME_type)
            self.intent2 = Intent.createChooser(self.intent,
                                                JString(self.msg))
            mActivity.startActivityForResult(self.intent2, self.REQUEST_CODE)
            Clock.schedule_once(self.begone_you_black_screen)
        except Exception as e:
            print('Pykivdroid ERROR : Picker.pick_file():\n' + str(e))        

    def intent_callback(self, requestCode, resultCode, intent):
        if resultCode and requestCode == self.REQUEST_CODE:
            try:
                if intent:
                    self.callback(intent.getData())
            except Exception as e:
                print('Pykivdroid ERROR Picker.intent_callback():\n' + str(e))        


   #updating kivy window view port to avoid black screen
    def begone_you_black_screen(self,dt):
        Window.update_viewport()
                
     



from pykivdroid import mActivity,WindowManagerNLayoutParams,Window,run_on_ui_thread,View

@run_on_ui_thread
def set_full_screen():
    return mActivity.getWindow().getDecorView().setSystemUiVisibility( 
                   View.SYSTEM_UI_FLAG_FULLSCREEN 
                   |View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN 
                   | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY  
                   | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                   | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION)
    




from pykivdroid import mActivity,orientation_dict,Logger

def get_orientation():
    return mActivity.getResources().getConfiguration().orientation

def set_orientation(mode='all'):
        try:
            if mode in orientation_dict:
                mActivity.setRequestedOrientation(orientation_dict[mode])
        except Exception as e:
            Logger.error("Pykivdroid: "+str(e))




from pykivdroid import SettingsNSystem,mActivity

'''
Date 28/May/2021

Required Permissions:
    WRITE_SETTINGS
'''
def get_brightness():
    SettingsNSystem.putInt(mActivity.getContentResolver(),
        SettingsNSystem.SCREEN_BRIGHTNESS_MODE,
        SettingsNSystem.SCREEN_BRIGHTNESS_MODE_MANUAL)
    cr_level = SettingsNSystem.getInt(mActivity.getContentResolver(),SettingsNSystem.SCREEN_BRIGHTNESS)
    return (cr_level / 255.) * 100

def set_brightness(level):
        SettingsNSystem.putInt(mActivity.getContentResolver(),SettingsNSystem.SCREEN_BRIGHTNESS,(level / 100.) * 255)








from kivy.uix.widget import Widget
from kivy.properties import BooleanProperty
from kivy.utils import platform
from pykivdroid import mActivity,VideoView,LinearLayout,ViewGroupNLayoutParams,MediaController,Gravity,run_on_ui_thread



class AndroidVideoView(Widget):
    videoview=None
    controller=BooleanProperty(True)
    def __init__(self,uri=None,**k):
        super().__init__(**k)


        

        self._init()
        
        
       
    
    @run_on_ui_thread        
    def _init(self,):
        #print("aaaaaaaaaa")
        if platform=="android":
            #print("aa")
            #mActivity = PythonActivity.mActivity 
            self.videoview=VideoView(mActivity)
            self.videoview.setMediaController(MediaController(mActivity))
            #self.videoview.setVideoURI(uri)
            self.layout = LinearLayout(mActivity)
            self.layout.setOrientation(LinearLayout.VERTICAL)
            self.layout.setGravity(Gravity.CENTER)
            self.layout.addView(self.videoview, self.width, self.height)
            mActivity.addContentView(self.layout, ViewGroupNLayoutParams(-1,-1))
            #self.videoview.start()

    @run_on_ui_thread
    def on_size(self, instance, size):
        if self.videoview:
        
            params = self.layout.getLayoutParams()
            params.width = self.width
            params.height = self.height
            self.layout.setLayoutParams(params)
            param = self.videoview.getLayoutParams()
            param.width = self.width
            param.height = self.height
            self.videoview.setLayoutParams(param)


    @run_on_ui_thread
    def start(self,):
        if self.videoview:
            self.videoview.setZOrderMediaOverlay(True);

            self.videoview.setZOrderOnTop(True)
            self.videoview.start()

    @run_on_ui_thread
    def pause(self,):
        if self.videoview:
            self.videoview.pause()

    @run_on_ui_thread
    def resume(self,):
        if self.videoview:
            self.videoview.resume()
    @run_on_ui_thread
    def stop_playback(self,):
        if self.videoview:
            self.videoview.stopPlayback()
    @run_on_ui_thread
    def suspend(self,):
        if self.videoview:
            self.videoview.suspend()
    
    @run_on_ui_thread
    def seek_to(self,ms):
        if self.videoview:
            self.videoview.seekTo(ms)





    @run_on_ui_thread
    def set_video_uri(self,uri):
        if self.videoview:
            self.videoview.setVideoURI(uri)
    @run_on_ui_thread
    def set_video_path(self,path):
        if self.videoview:
            self.videoview.setVideoPath(path)
    






     ########bools        
    @run_on_ui_thread
    def is_playing(self,):
        if self.videoview:
            return self.videoview.isPlaying()
    @run_on_ui_thread
    def get_duration(self,):
        if self.videoview:
            return self.videoview.getDuration()
    @run_on_ui_thread
    def get_current_position(self,):
        if self.videoview:
            return self.videoview.getCurrentPosition()
    @run_on_ui_thread
    def get_audio_session_id(self,):
        if self.videoview:
            return self.videoview.getAudioSessionId()



    @run_on_ui_thread
    def can_pause(self,):
        if self.videoview:
            return self.videoview.canPause()
    @run_on_ui_thread
    def can_seek_backward(self,):
        if self.videoview:
            return self.videoview.canSeekBackward()
    @run_on_ui_thread
    def can_seek_forward(self,):
        if self.videoview:
            return self.videoview.canSeekForward()
    @run_on_ui_thread
    def can_pause(self,):
        if self.videoview:
            return self.videoview.canPause()










 



from setuptools import setup, find_packages
import codecs
import os

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.1'
DESCRIPTION = 'Python tools to control android'
LONG_DESCRIPTION = 'Python tools to control android by python'

# Setting up
setup(
    name="pykivdroid",
    version=VERSION,
    author="SK SAHIL",
    #author_email="",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['kivy','pyjnius','android'],
    keywords=['python', 'kivy', 'android', 'pyjnius'],
    ujrl='https://github.com/Sahil-pixel/Pykivdroid',
    classifiers=[]
) 
